﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Problems.Instances;
using HeuristicLab.Problems.Instances.QAPLIB;
using HeuristicLab.Problems.QuadraticAssignment;

namespace ProblemInstanceIdentifier {
  class Program {
    static void Main(string[] args) {
      var sync = new object();

      var qapProviders = ApplicationManager.Manager.GetInstances<ProblemInstanceProvider<QAPData>>().ToList();
      var instances = new List<QuadraticAssignmentProblem>();
      foreach (var provider in qapProviders) {
        if (provider is TaillardQAPInstanceProvider) continue;
        Parallel.ForEach(provider.GetDataDescriptors(), desc => {
          if (desc.Name == "esc16f") return;
          var qapData = provider.LoadData(desc);
          if (qapData.Dimension < 15 || qapData.Dimension > 150) return;
          var qap = new QuadraticAssignmentProblem();
          qap.Load(qapData);
          lock (sync) {
            instances.Add(qap);
          }
        });
      }

      Console.WriteLine("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6:F2}\t{7:F2}", "Repetition", "KB-Paths", "Exp-Paths", "Cls-Hits", "Exact-Hits", "No-Hits", "Cls-Rank", "Exact-Rank");
      var paths = new[] { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 20, 30, 40, 50 };
      for (var r = 0; r < 20; r++) {
        Parallel.ForEach(paths, kbPaths => {
          foreach (var expPaths in paths) {
            if (expPaths > kbPaths) continue;
            var kb = new List<InstanceDescriptor>();
            var exp = new List<InstanceDescriptor>();
            foreach (var qap in instances) {
              kb.Add(InstanceDescriptor.FromPathRelinkingAnalysis(qap, kbPaths, null));
              exp.Add(InstanceDescriptor.FromPathRelinkingAnalysis(qap, expPaths, null));
            }
            int exactCount = 0, clsCount = 0, missedCount = 0;
            int exactRank = 0, clsRank = 0;
            foreach (var e in exp) {
              var ordered = kb.OrderBy(x => x.CalculateSimilarity(e)).ToList();
              var bestMatch = ordered.First();
              if (bestMatch.Cls == e.Cls) {
                clsCount++;
                if (bestMatch.Name == e.Name) exactCount++;
              } else missedCount++;
              clsRank += ordered.FindIndex((id) => id.Cls == e.Cls) + 1;
              exactRank += ordered.FindIndex((id) => id.Name == e.Name) + 1;
            }
            lock (sync) {
              Console.WriteLine("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6:F2}\t{7:F2}", r, kbPaths, expPaths, clsCount, exactCount, missedCount, clsRank / (double)exp.Count, exactRank / (double)exp.Count);
            }
          }
        });
      }
    }
  }
}
